package main

import (
	"flag"
	"fmt"
	"io"
	"log"
	"os"
	"path/filepath"

	"github.com/nwaples/rardecode"
)

type Mode int

const (
	List Mode = iota
	Extract
)

var modes = map[string]Mode{
	"t": List,
	"x": Extract,
}

var (
	password = flag.String("password", "", "rar archive password")
)

func usage() {
	log.Fatalf("usage: unrar [tx] file.rar")
}

func main() {
	flag.Parse()

	args := flag.Args()
	if len(args) != 2 {
		usage()
	}

	if len(args[0]) != 1 {
		usage()
	}

	mode := modes[args[0]]

	f, err := os.Open(args[1])
	if err != nil {
		log.Fatal(err)
	}

	defer f.Close()

	rar, err := rardecode.NewReader(f, *password)
	if err != nil {
		log.Fatal(err)
	}

	for {
		hdr, err := rar.Next()
		if err == io.EOF {
			// end of tar archive
			break
		}
		if err != nil {
			log.Fatalln(err)
		}

		switch mode {
		case List:
			fmt.Printf("%o %d %s %s\n", hdr.Attributes, hdr.UnPackedSize, hdr.ModificationTime, hdr.Name)

		case Extract:
			fmt.Printf("%o %d %s %s\n", hdr.Attributes, hdr.UnPackedSize, hdr.ModificationTime, hdr.Name)

			if hdr.IsDir {
				// we'll handle this when making files
				continue
			}

			dirpath := filepath.Dir(hdr.Name)
			if dirpath != "." {
				if _, err := os.Stat(dirpath); err != nil {
					if err := os.MkdirAll(dirpath, 0775); err != nil {
						log.Fatalf("failed to create %q: %v", dirpath, err)
					}
				}
			}

			cf, err := os.Create(hdr.Name)
			if err != nil {
				log.Fatalf("failed to create %q: %v", hdr.Name, err)
			}

			_, err = io.Copy(cf, rar)
			if err != nil {
				log.Fatalf("failed to write to %q: %v", hdr.Name, err)
			}

			cf.Close()
		}
	}

}

